Package: marginaleffects
Title: Predictions, Comparisons, Slopes, Marginal Means, and Hypothesis
        Tests
Version: 0.32.0
Authors@R: 
    c(person(given = "Vincent",
             family = "Arel-Bundock",
             role = c("aut", "cre", "cph"),
             email = "vincent.arel-bundock@umontreal.ca",
             comment = c(ORCID = "0000-0003-2042-7063")),
      person(given = "Noah",
             family = "Greifer",
             role = "ctb",
             email = "noah.greifer@gmail.com",
             comment = c(ORCID = "0000-0003-3067-7154")),
      person("Etienne", "Bacher",
             email = "etienne.bacher@protonmail.com",
             role = "ctb",
             comment = c(ORCID = "0000-0002-9271-5075")),
      person(given = "Grant",
             family = "McDermott",
             role = c("ctb"),
             comment = c(ORCID = "0000-0001-7883-8573")),
      person(given = "Andrew",
             family = "Heiss",
             role = c("ctb"),
             comment = c(ORCID = "0000-0002-3948-3914"))
             )
Description: Compute and plot predictions, slopes, marginal means, and comparisons (contrasts, risk ratios, odds, etc.) for over 100 classes of statistical and machine learning models in R. Conduct linear and non-linear hypothesis tests, or equivalence tests. Calculate uncertainty estimates using the delta method, bootstrapping, or simulation-based inference. Details can be found in Arel-Bundock, Greifer, and Heiss (2024) <doi:10.18637/jss.v111.i09>.
License: GPL (>= 3)
Copyright: inst/COPYRIGHTS
Encoding: UTF-8
URL: https://marginaleffects.com/
BugReports: https://github.com/vincentarelbundock/marginaleffects/issues
RoxygenNote: 7.3.3
Depends: R (>= 3.6.0)
Imports: backports, checkmate, data.table (>= 1.16.4), generics,
        Formula, insight (>= 1.3.0), methods, rlang
Suggests: AER, Amelia, afex, aod, arrow, bayestestR, bench, betareg,
        BH, bife, biglm, blme, boot, brglm2, brms, brmsmargins, broom,
        car, carData, causaldata, clarify, cjoint, cobalt, collapse,
        conflicted, countrycode, covr, crch, DALEXtra, DCchoice,
        dbarts, DirichletReg, distributional, dfidx, dplyr, emmeans,
        equivalence, estimatr, fixest, flexsurv, fmeffects, fontquiver,
        future, future.apply, fwb, gam, gamlss, gamlss.dist, geepack,
        ggdist, ggokabeito, ggplot2, ggrepel, glmmTMB, glmtoolbox,
        glmx, haven, here, itsadug, ivreg, kableExtra, lme4, lmerTest,
        logistf, magrittr, margins, MatchIt, MASS, mclogit, MCMCglmm,
        mhurdle, missRanger, mgcv, mice, miceadds, mlogit, mlr3verse,
        modelbased, modelsummary, multcomp, mvgam, mvtnorm,
        nanoparquet, nlme, nnet, numDeriv, nycflights13, optmatch,
        ordbetareg, ordinal, parameters, parsnip, partykit, patchwork,
        pkgdown, phylolm, pbapply, plm, polspline, posterior, probably,
        pscl, purrr, quantreg, quantregForest, Rchoice, REndo,
        rcmdcheck, Rdatasets, remotes, reticulate, rmarkdown, rms,
        robust, robustbase, robustlmm, rsample, rstanarm, rstantools,
        rstpm2, rstudioapi, rsvg, sampleSelection, sandwich, scam,
        spelling, speedglm, splines, survey, survival, svglite,
        systemfit, systemfonts, tibble, tictoc, tidymodels, tidyr,
        tidyverse, tinysnapshot (>= 0.0.7), tinytable (>= 0.6.1),
        tinytest, titanic, truncreg, tsModel, withr, workflows, yaml,
        xgboost, altdoc, knitr, quarto
Collate: 'add_hypothesis.R' 'add_rowid.R' 'attributes.R' 'autodiff.R'
        'backtransform.R' 'broom.R' 'by.R' 'class.R' 'comparisons.R'
        'components.R' 'construct_call.R' 'datagrid.R' 'equivalence.R'
        'get_ci.R' 'get_coef.R' 'get_comparisons.R'
        'get_comparisons_data.R' 'get_comparisons_data_character.R'
        'get_comparisons_data_factor.R'
        'get_comparisons_data_logical.R'
        'get_comparisons_data_numeric.R' 'get_dataset.R'
        'get_degrees_of_freedom.R' 'get_draws.R' 'get_group_names.R'
        'get_hypotheses.R' 'get_jacobian.R' 'get_labels.R'
        'get_model_matrix.R' 'get_modeldata.R' 'get_predict.R'
        'get_predictions.R' 'get_se_delta.R' 'get_vcov.R'
        'github_issue.R' 'hush.R' 'hypotheses.R' 'hypotheses_joint.R'
        'hypothesis.R' 'hypothesis_formula.R' 'hypothesis_function.R'
        'hypothesis_matrix.R' 'hypothesis_string.R' 'imputation.R'
        'inferences.R' 'inferences_boot.R' 'inferences_conformal.R'
        'inferences_conformal_cv.R' 'inferences_conformal_full.R'
        'inferences_conformal_quantile.R'
        'inferences_conformal_split.R' 'inferences_fwb.R'
        'inferences_rsample.R' 'inferences_simulation.R'
        'matrix_apply.R' 'mean_or_mode.R' 'methods.R' 'set_coef.R'
        'methods_AER.R' 'sanity_model.R' 'methods_DirichletReg.R'
        'methods_MASS.R' 'methods_MCMCglmm.R' 'methods_Rchoice.R'
        'methods_WeightIt.R' 'methods_aaa.R' 'methods_afex.R'
        'methods_aod.R' 'methods_betareg.R' 'methods_bife.R'
        'methods_biglm.R' 'methods_nnet.R' 'methods_brglm2.R'
        'methods_brms.R' 'methods_crch.R' 'methods_dataframe.R'
        'methods_dbarts.R' 'methods_fixest.R' 'methods_flexsurv.R'
        'methods_gamlss.R' 'methods_glmmTMB.R' 'methods_glmtoolbox.R'
        'methods_glmx.R' 'methods_ivreg.R' 'methods_lme4.R'
        'methods_mclogit.R' 'methods_mgcv.R' 'methods_mhurdle.R'
        'methods_mice.R' 'methods_mlm.R' 'methods_mlogit.R'
        'methods_mlr3.R' 'methods_nlme.R' 'methods_ordinal.R'
        'methods_plm.R' 'methods_pscl.R' 'methods_quantreg.R'
        'methods_rms.R' 'methods_robustlmm.R' 'methods_rstanarm.R'
        'methods_rstpm2.R' 'methods_sampleSelection.R' 'methods_scam.R'
        'methods_survey.R' 'methods_survival.R' 'methods_systemfit.R'
        'methods_tidymodels.R' 'methods_tobit1.R' 'modelarchive.R'
        'multcomp.R' 'myTryCatch.R' 'package.R' 'pad.R' 'plot.R'
        'plot_build.R' 'plot_comparisons.R' 'plot_predictions.R'
        'plot_slopes.R' 'predictions.R' 'print.R' 'prune.R' 'refit.R'
        'sanitize_comparison.R' 'sanitize_condition.R'
        'sanitize_conf_level.R' 'sanitize_hypothesis_formula.R'
        'sanitize_interaction.R' 'sanitize_newdata.R'
        'sanitize_numderiv.R' 'sanitize_reserved.R' 'sanitize_type.R'
        'sanitize_variables.R' 'sanitize_vcov.R' 'sanity.R'
        'sanity_by.R' 'sanity_dots.R' 'sanity_inferences.R'
        'sanity_multcomp.R' 'settings.R' 'slopes.R' 'sort.R'
        'tinytest.R' 'type_dictionary.R' 'unpack_matrix_cols.R'
        'utils.R' 'zzz.R'
Language: en-US
VignetteBuilder: knitr, quarto
NeedsCompilation: no
Packaged: 2026-02-14 02:05:39 UTC; vincent
Author: Vincent Arel-Bundock [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-2042-7063>),
  Noah Greifer [ctb] (ORCID: <https://orcid.org/0000-0003-3067-7154>),
  Etienne Bacher [ctb] (ORCID: <https://orcid.org/0000-0002-9271-5075>),
  Grant McDermott [ctb] (ORCID: <https://orcid.org/0000-0001-7883-8573>),
  Andrew Heiss [ctb] (ORCID: <https://orcid.org/0000-0002-3948-3914>)
Maintainer: Vincent Arel-Bundock <vincent.arel-bundock@umontreal.ca>
Repository: CRAN
Date/Publication: 2026-02-14 06:40:08 UTC
Built: R 4.6.0; ; 2026-04-26 09:59:28 UTC; unix
