# Generated by roxygen2: do not edit by hand

S3method(coef,comparisons)
S3method(coef,hypotheses)
S3method(coef,predictions)
S3method(coef,slopes)
S3method(components,comparisons)
S3method(components,hypotheses)
S3method(components,marginaleffects)
S3method(components,predictions)
S3method(components,slopes)
S3method(df.residual,comparisons)
S3method(df.residual,predictions)
S3method(df.residual,slopes)
S3method(get_autodiff_args,glm)
S3method(get_autodiff_args,ivreg)
S3method(get_autodiff_args,lm)
S3method(get_coef,DirichletRegModel)
S3method(get_coef,afex_aov)
S3method(get_coef,betareg)
S3method(get_coef,bracl)
S3method(get_coef,brmsfit)
S3method(get_coef,brmultinom)
S3method(get_coef,comparisons)
S3method(get_coef,data.frame)
S3method(get_coef,default)
S3method(get_coef,gam)
S3method(get_coef,gamlss)
S3method(get_coef,glmgee)
S3method(get_coef,glmmTMB)
S3method(get_coef,lmerMod)
S3method(get_coef,lmerModLmerTest)
S3method(get_coef,mblogit)
S3method(get_coef,merMod)
S3method(get_coef,mlm)
S3method(get_coef,multinom)
S3method(get_coef,multinom_weightit)
S3method(get_coef,nls)
S3method(get_coef,polr)
S3method(get_coef,predictions)
S3method(get_coef,scam)
S3method(get_coef,selection)
S3method(get_coef,slopes)
S3method(get_coef,svyolr)
S3method(get_coef,systemfit)
S3method(get_coef,workflow)
S3method(get_group_names,bracl)
S3method(get_group_names,brmsfit)
S3method(get_group_names,clm)
S3method(get_group_names,default)
S3method(get_group_names,hurdle)
S3method(get_group_names,mblogit)
S3method(get_group_names,mlm)
S3method(get_group_names,multinom)
S3method(get_group_names,polr)
S3method(get_group_names,svyolr)
S3method(get_mean_or_mode,character)
S3method(get_mean_or_mode,data.frame)
S3method(get_mean_or_mode,default)
S3method(get_mean_or_mode,factor)
S3method(get_mean_or_mode,logical)
S3method(get_model_matrix,default)
S3method(get_predict,DirichletRegModel)
S3method(get_predict,Learner)
S3method(get_predict,MCMCglmm)
S3method(get_predict,afex_aov)
S3method(get_predict,aft)
S3method(get_predict,bart)
S3method(get_predict,betareg)
S3method(get_predict,bife)
S3method(get_predict,biglm)
S3method(get_predict,brmsfit)
S3method(get_predict,brmultinom)
S3method(get_predict,clm)
S3method(get_predict,coxph)
S3method(get_predict,crch)
S3method(get_predict,default)
S3method(get_predict,fixest)
S3method(get_predict,flexsurvreg)
S3method(get_predict,gamlss)
S3method(get_predict,glimML)
S3method(get_predict,glm)
S3method(get_predict,glmgee)
S3method(get_predict,glmmPQL)
S3method(get_predict,glmmTMB)
S3method(get_predict,gsm)
S3method(get_predict,hxlr)
S3method(get_predict,lda)
S3method(get_predict,lm)
S3method(get_predict,lmerMod)
S3method(get_predict,lmerModLmerTest)
S3method(get_predict,lrm)
S3method(get_predict,mblogit)
S3method(get_predict,merMod)
S3method(get_predict,mhurdle)
S3method(get_predict,mlogit)
S3method(get_predict,model_fit)
S3method(get_predict,multinom)
S3method(get_predict,ols)
S3method(get_predict,orm)
S3method(get_predict,polr)
S3method(get_predict,pstpm2)
S3method(get_predict,rlmerMod)
S3method(get_predict,rms)
S3method(get_predict,rq)
S3method(get_predict,stanreg)
S3method(get_predict,stpm2)
S3method(get_predict,svyglm)
S3method(get_predict,svyolr)
S3method(get_predict,systemfit)
S3method(get_predict,tobit1)
S3method(get_predict,workflow)
S3method(get_vcov,Learner)
S3method(get_vcov,MCMCglmm)
S3method(get_vcov,afex_aov)
S3method(get_vcov,aft)
S3method(get_vcov,bart)
S3method(get_vcov,biglm)
S3method(get_vcov,brmsfit)
S3method(get_vcov,comparisons)
S3method(get_vcov,default)
S3method(get_vcov,gamlss)
S3method(get_vcov,glimML)
S3method(get_vcov,glmmTMB)
S3method(get_vcov,gsm)
S3method(get_vcov,hypotheses)
S3method(get_vcov,lda)
S3method(get_vcov,mhurdle)
S3method(get_vcov,model_fit)
S3method(get_vcov,orm)
S3method(get_vcov,predictions)
S3method(get_vcov,pstpm2)
S3method(get_vcov,scam)
S3method(get_vcov,slopes)
S3method(get_vcov,stpm2)
S3method(get_vcov,systemfit)
S3method(get_vcov,workflow)
S3method(glance,comparisons)
S3method(glance,hypotheses)
S3method(glance,marginaleffects_mids)
S3method(glance,predictions)
S3method(glance,slopes)
S3method(knitr::knit_print, marginaleffects)
S3method(knitr::knit_print,comparisons)
S3method(knitr::knit_print,hypotheses)
S3method(knitr::knit_print,predictions)
S3method(knitr::knit_print,slopes)
S3method(plot,comparisons)
S3method(plot,predictions)
S3method(plot,slopes)
S3method(print,comparisons)
S3method(print,hypotheses)
S3method(print,marginaleffects)
S3method(print,predictions)
S3method(print,slopes)
S3method(prune,comparisons)
S3method(prune,hypotheses)
S3method(prune,marginaleffects)
S3method(prune,predictions)
S3method(prune,slopes)
S3method(refit,comparisons)
S3method(refit,hypotheses)
S3method(refit,marginaleffects)
S3method(refit,predictions)
S3method(refit,slopes)
S3method(sanitize_model_specific,DirichletRegModel)
S3method(sanitize_model_specific,Learner)
S3method(sanitize_model_specific,bart)
S3method(sanitize_model_specific,brmsfit)
S3method(sanitize_model_specific,gamlss)
S3method(sanitize_model_specific,lrm)
S3method(sanitize_model_specific,mblogit)
S3method(sanitize_model_specific,merMod)
S3method(sanitize_model_specific,ols)
S3method(sanitize_model_specific,orm)
S3method(sanitize_model_specific,svyglm)
S3method(sanitize_model_specific,svyolr)
S3method(set_coef,DirichletRegModel)
S3method(set_coef,afex_aov)
S3method(set_coef,aft)
S3method(set_coef,betareg)
S3method(set_coef,crch)
S3method(set_coef,data.frame)
S3method(set_coef,default)
S3method(set_coef,flexsurvreg)
S3method(set_coef,gamlss)
S3method(set_coef,glimML)
S3method(set_coef,glm)
S3method(set_coef,glmgee)
S3method(set_coef,glmmPQL)
S3method(set_coef,glmmTMB)
S3method(set_coef,glmx)
S3method(set_coef,gsm)
S3method(set_coef,hetprob)
S3method(set_coef,hurdle)
S3method(set_coef,hxlr)
S3method(set_coef,ivpml)
S3method(set_coef,lm)
S3method(set_coef,lme)
S3method(set_coef,lmerMod)
S3method(set_coef,lmerModLmerTest)
S3method(set_coef,merMod)
S3method(set_coef,mlm)
S3method(set_coef,model_fit)
S3method(set_coef,multinom)
S3method(set_coef,nls)
S3method(set_coef,polr)
S3method(set_coef,pstpm2)
S3method(set_coef,rlmerMod)
S3method(set_coef,scam)
S3method(set_coef,selection)
S3method(set_coef,stpm2)
S3method(set_coef,survreg)
S3method(set_coef,svyolr)
S3method(set_coef,systemfit)
S3method(set_coef,tobit)
S3method(set_coef,workflow)
S3method(set_coef,zeroinfl)
S3method(tidy,comparisons)
S3method(tidy,hypotheses)
S3method(tidy,marginaleffects_mids)
S3method(tidy,predictions)
S3method(tidy,slopes)
S3method(vcov,comparisons)
S3method(vcov,hypotheses)
S3method(vcov,predictions)
S3method(vcov,slopes)
export(autodiff)
export(avg_comparisons)
export(avg_predictions)
export(avg_slopes)
export(comparisons)
export(components)
export(datagrid)
export(expect_comparisons)
export(expect_hypotheses)
export(expect_margins)
export(expect_predictions)
export(expect_slopes)
export(get_coef)
export(get_dataset)
export(get_draws)
export(get_group_names)
export(get_model_matrix)
export(get_predict)
export(get_vcov)
export(glance)
export(hypotheses)
export(inferences)
export(plot_comparisons)
export(plot_predictions)
export(plot_slopes)
export(posterior_draws)
export(predictions)
export(prune)
export(refit)
export(set_coef)
export(slopes)
export(tidy)
import(data.table)
importFrom(generics,components)
importFrom(generics,glance)
importFrom(generics,prune)
importFrom(generics,tidy)
