github_issue <- function() {
    msg <- insight::format_string(
        "Please report this error along with a minimal reproducible example using
    publicly available data on Github:

    https://github.com/vincentarelbundock/marginaleffects/issues
    "
    )
    return(msg)
}
