# https://stackoverflow.com/questions/9439256/how-can-i-handle-r-cmd-check-no-visible-binding-for-global-variable-notes-when#comment20826625_12429344
# 2012 hadley says "globalVariables is a hideous hack and I will never use it"
# 2014 hadley updates his own answer with globalVariables as one of "two solutions"
#' @import data.table
utils::globalVariables(c(
    ".",
    "..by",
    "..bycols",
    "..cl",
    "..col_newdata",
    "..col_x",
    "..cols",
    "..hypothesis_by",
    "..idx",
    "..idx1",
    "..idx2",
    "..idx_by",
    "..idx_hi",
    "..idx_lo",
    "..idx_or",
    "..keepers",
    "..lab",
    "..lab_cols",
    "..label_columns",
    "..nd",
    "..nonfocal",
    "..tmp",
    "..to_keep",
    "..variables_grid",
    "..w",
    ".data",
    "N",
    "aa",
    "chunk",
    "comparison",
    "comparison_idx",
    "condition_variable",
    "conf.high",
    "conf.low",
    "conf_int",
    "conf_level",
    "contrast",
    "draw",
    "dydx",
    "elast",
    "eps",
    "estimate",
    "estimates",
    "formatBicLL",
    "gn",
    "group",
    "hi",
    "lo",
    "logLik",
    "m",
    "mAD",
    "marginaleffects_contrast_hi",
    "marginaleffects_contrast_lo",
    "marginaleffects_function",
    "marginaleffects_group_id",
    "marginaleffects_internal_label",
    "marginaleffects_internal_rowid",
    "marginaleffects_lab",
    "marginaleffects_ter",
    "marginaleffects_unique_labels",
    "marginaleffects_wts_internal",
    "marginalmean",
    "model",
    "p.value",
    "part",
    "patterns",
    "position",
    "pred_hi",
    "pred_lo",
    "predicted",
    "predicted_hi",
    "predicted_lo",
    "predicted_or",
    "results",
    "rowid",
    "rowidcf",
    "rowidunique",
    "section",
    "stars_note",
    "statistic",
    "statistic1",
    "std.error",
    "term",
    "ticks",
    "tmp_grp",
    "tmp_idx",
    "value",
    "vcov",
    "where",
    "wts"
))
