% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tinytest.R
\name{expect_hypotheses}
\alias{expect_hypotheses}
\title{\code{tinytest} helper}
\usage{
expect_hypotheses(object, se = TRUE, n_row = NULL, n_col = NULL, ...)
}
\description{
\code{tinytest} helper
}
\keyword{internal}
