% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tinytest.R
\name{expect_slopes}
\alias{expect_slopes}
\title{\code{tinytest} helper}
\usage{
expect_slopes(object, n_unique = NULL, pct_na = 5, se = TRUE, ...)
}
\description{
\code{tinytest} helper
}
\keyword{internal}
